use_inline_resources

def whyrun_supported?
  true
end

action :create do

  if @current_resource.exists == 0
    Chef::Log.info "#{ @current_resource } already exists - nothing to do."
  elsif @current_resource.exists == 1
    converge_by("Updating #{ @current_resource }") do
      Chef::Log.info "#{ @current_resource } Updating netgroup."
      updating_netgroup(@current_resource)
    end
  else
    converge_by("Create #{ @current_resource }") do
      Chef::Log.info "#{ @current_resource } Creating netgroup."
      create_netgroup(@current_resource)
    end
  end
end

action :remove do
  if @current_resource.exists == 0 || @current_resource.exists == 1
    converge_by("#{ @current_resource } exists - removing.") do
      Chef::Log.info "#{ @current_resource } exists - removing."
      remove_netgroup(@current_resource)
    end
  end
end

action :manage do
  Chef::Log.info "#{ @current_resource } managed by chef. Set the action to create to update the netgroup"
end

def load_current_resource
  # Retreive values and build the netgroup
  @current_resource = Chef::Resource::Netgroup.new(@new_resource.name)
  @current_resource.name(@new_resource.name)
  @current_resource.users(@new_resource.users)
  @current_resource.groups(@new_resource.groups)
  @current_resource.netgroup = build_current_resource_netgroup_entry(@current_resource)
  @current_resource.exists = netgroup_exists_test(@current_resource)

  Chef::Log.info "Resource name = #{@current_resource.name}"
end

def remove_netgroup(resource)
  text = ::File.read('/etc/netgroup')
  text = text.gsub(/^#{resource.name.to_s}.*$\r?\n?/, '')

  ::File.open('/etc/netgroup', 'w+') do |f1|
    f1.puts(text.to_s)
  end
end

def updating_netgroup(resource)
  text = ::File.read('/etc/netgroup')
  text = text.gsub(/^#{resource.name.to_s}.*$/, resource.netgroup.to_s)

  ::File.open('/etc/netgroup', 'w+') do |f1|
    f1.puts(text.to_s)
  end
end

def create_netgroup(resource)
  ::File.open('/etc/netgroup', 'a') do |f1|
    f1.puts(resource.netgroup)
  end
end

# return a string that can be used in regex with special characters escaped
def escape_string(str)
  Regexp.escape(str).gsub(/([:~!<>="])/, '\\\\\1')
end

def netgroup_exists_test(resource)
  # transform the netgroup so it can be used in regex
  escaped_resource_netgroup = escape_string(resource.netgroup.to_s)

  # read in the netgroup file
  text = ::File.read('/etc/netgroup')

  # if the netgroup and all elements already exist do not update
  if text.to_s.match(/^#{resource.name}/) &&  text.to_s.match(/^#{escaped_resource_netgroup}$/)
    Chef::Log.info "Netgroup #{resource.name} already exists and is up to date."
    return 0
  # if the netgroup exists but elements are different update the net group
  elsif text.to_s.match(/^#{resource.name}/) && !text.to_s.match(/^#{escaped_resource_netgroup}$/)
    Chef::Log.info "Netgroup #{resource.name} already exists but is out of date."
    return 1
  # else the element netgroup is missing
  else
    Chef::Log.info "Netgroup #{resource.name} does not exist and needs to be created."
    return 2
  end
end

# build the netgroup element of the instance
# groups should be first the users
def build_current_resource_netgroup_entry(resource)
  netgroup = "#{resource.name} "
  resource.groups.each do |group|
    netgroup += "#{group} "
  end

  resource.users.each do |user|
    netgroup += "(,#{user},) "
  end
  netgroup = netgroup.sub(/\s$/, '')
end
